import java.awt.*;
import java.awt.image.*;
import java.awt.event.*;
import javax.swing.*;

/** Class for building the GUI for playing video files and viewing r-frames after the segmentation process
 * 
 *  @version 1.0 01.06.06
 *  @author Vegard Kosmo
 *  @see Videosegmenter
 *  @see Button
 *
 */
public class GUI implements ActionListener{
	// Main application reference
	Videosegmenter main;
	
	// Button reference
	private Button button;
	
	// JFrame variables
	private JFrame videoWindow;
	public JFrame shotWindow;
	private String videoTitle, shotTitle;
	private Point videoPlacement, shotPlacement;
	public JDesktopPane desktopPane;
	public JPanel imagePanel;
	public GridLayout gridLayout;
	private JScrollPane scroll;
	private JScrollBar scrollBar;
		
	// JMenu variables
	private JMenuBar videoMenu;
	private JMenu file, algorithm, threshold;
	private JMenuItem open, exit;
	private JCheckBoxMenuItem algorithm1, algorithm2, algorithm3, threshold1, threshold2, threshold3;
	
	/** Standard constructor to build the GUI
	 *  
	 *  @param main Main class reference
	 */
	public GUI(Videosegmenter main){
		this.main = main;		
	}
	
	/** Method to build the video window
	 * 
	 *  @param size The size of the video window
	 *  @return videoWindow Returns the generated video window
	 */
	public JFrame videoWindow(Dimension size){
		int width = (int)size.getWidth();
		int height = (int)size.getHeight();
		videoTitle = "Video segmenter";
		videoPlacement = new Point(0, 0);
		videoWindow = new JFrame();
		videoWindow.setSize(width, height);
		videoWindow.setTitle(videoTitle);
		videoWindow.setLocation(videoPlacement);
		videoWindow.setResizable(false);
		desktopPane = new JDesktopPane();
		desktopPane.setDoubleBuffered(true);
		desktopPane.setBackground(Color.BLACK);
		videoWindow.getContentPane().add("Center", desktopPane);
					
		// Method to close the window
		videoWindow.addWindowListener( 
			new WindowAdapter() {
				public void windowClosing(WindowEvent we) {
					System.exit(0);
				}
			} 
		);
		
		return videoWindow;
	}
	
	/** Method to build the shot viewer window 
	 * 
	 *  @return shotWindow Returns the generated shot window
	 */
	public JFrame shotWindow(){
		shotTitle = "Video shot viewer";
		shotPlacement = new Point(400, 0);
		shotWindow = new JFrame();
		shotWindow.setBackground(Color.white);
		shotWindow.setTitle(shotTitle);
		shotWindow.setLocation(shotPlacement);
		shotWindow.setResizable(false);
		
		// Sets the layout for the video shot window
		gridLayout = new GridLayout();
		gridLayout.setColumns(7);
		gridLayout.setHgap(2);
		gridLayout.setVgap(2);
		
		// Sets the image panel for the video shot window
		imagePanel = new JPanel();
		imagePanel.setLayout(gridLayout);
		imagePanel.setBackground(Color.WHITE);
				
		scrollBar = new JScrollBar();
		scrollBar.setEnabled(true);
		scroll = new JScrollPane(imagePanel);
		scroll.getVerticalScrollBar().setUnitIncrement(20);
		scroll. setVerticalScrollBarPolicy(ScrollPaneConstants.VERTICAL_SCROLLBAR_AS_NEEDED);
		scroll. setHorizontalScrollBarPolicy(ScrollPaneConstants.HORIZONTAL_SCROLLBAR_NEVER);
		scroll.add(scrollBar);
		shotWindow.setContentPane(scroll);
						
		// Method to close the window
		shotWindow.addWindowListener( 
			new WindowAdapter() {
				public void windowClosing(WindowEvent we){					
					System.exit(0);
				}
			} 
		);
		
		return shotWindow;
	}
	
	/** Method to build the video window menu 
	 * 
	 *  @return videoMenu Returns the video menu
	 */
	public JMenuBar videoMenu(){
		videoMenu = new JMenuBar();
		
		// Menu
		file = new JMenu("File");
		algorithm = new JMenu("Algorithm");
		threshold = new JMenu("Threshold");
		
		// The File menu
		open = new JMenuItem("Open video file");
		open.setEnabled(true);
		open.addActionListener(this);
		
		exit = new JMenuItem("Exit");
		exit.setEnabled(true);
		exit.addActionListener(this);
		
		// The algorithm menu
		algorithm1 = new JCheckBoxMenuItem("Global color distribution");
		algorithm1.setEnabled(true);
		algorithm1.setSelected(true);
		algorithm1.addActionListener(this);
		
		algorithm2 = new JCheckBoxMenuItem("Local color distribution");
		algorithm2.setEnabled(true);
		algorithm2.setSelected(false);
		algorithm2.addActionListener(this);
		
		algorithm3 = new JCheckBoxMenuItem("Global edge chasing");
		algorithm3.setEnabled(true);
		algorithm3.setSelected(false);
		algorithm3.addActionListener(this);
		
		// The threshold menu
		threshold1 = new JCheckBoxMenuItem("Low");
		threshold1.setEnabled(true);
		threshold1.setSelected(false);
		threshold1.addActionListener(this);
		
		threshold2 = new JCheckBoxMenuItem("Medium");
		threshold2.setEnabled(true);
		threshold2.setSelected(true);
		threshold2.addActionListener(this);
		
		threshold3 = new JCheckBoxMenuItem("High");
		threshold3.setEnabled(true);
		threshold3.setSelected(false);
		threshold3.addActionListener(this);
		
		// Add to menu
		file.add(open);
		file.add(exit);
		
		algorithm.add(algorithm1);
		algorithm.add(algorithm2);
		algorithm.add(algorithm3);
		
		threshold.add(threshold1);
		threshold.add(threshold2);
		threshold.add(threshold3);
		
		videoMenu.add(file);
		videoMenu.add(algorithm);
		videoMenu.add(threshold);
				
		return videoMenu;
	}
	
	// Method to handle action requests	
	public void actionPerformed(ActionEvent ae){
		String command = ae.getActionCommand();
		
		// Events from the 'File' menu
		if(command.equals("Open video file")) {
    		main.openFile();
    	}
    	
    	else if(command.equals("Exit")) {
    		main.exit();
    	}
		
		// Events from the 'Algorithm' menu
    	else if(command.equals("Global color distribution")){
    		algorithm1.setSelected(true);
    		algorithm2.setSelected(false);
    		algorithm3.setSelected(false);
    		main.changeValues(1);
    		main.buildAlgorithm();
    	}
		
    	else if(command.equals("Local color distribution")){
    		algorithm1.setSelected(false);
    		algorithm2.setSelected(true);
    		algorithm3.setSelected(false);
    		main.changeValues(2);
    		main.buildAlgorithm();
    	}
		
    	else if(command.equals("Global edge chasing")){
    		algorithm1.setSelected(false);
    		algorithm2.setSelected(false);
    		algorithm3.setSelected(true);
    		main.changeValues(3);
    		main.buildAlgorithm();
    	}
		
		// Events from the 'Threshold' menu
    	else if(command.equals("Low")){
    		threshold1.setSelected(true);
    		threshold2.setSelected(false);
    		threshold3.setSelected(false);
    		main.changeThreshold(1);
    	}
		
    	else if(command.equals("Medium")){
    		threshold1.setSelected(false);
    		threshold2.setSelected(true);
    		threshold3.setSelected(false);
    		main.changeThreshold(2);
    	}
		
    	else if(command.equals("High")){
    		threshold1.setSelected(false);
    		threshold2.setSelected(false);
    		threshold3.setSelected(true);
    		main.changeThreshold(3);
    	}
	}
	
	/** Method that adds new shot images to the shot window
	 * 
	 *  @param resizedImageObject The resized image to be viewed in the shot window
	 */
	public void addImage(ResizedImageObject resizedImageObject){
		button = new Button(resizedImageObject.getFrameNumber(), main);
		button.addActionListener(button);
		ImageIcon imageIcon = new ImageIcon();
		imageIcon.setImage((BufferedImage)(resizedImageObject.getBufferedImage()));		
		button.setIcon(imageIcon);
		button.setBackground(Color.white);
		button.setPreferredSize(new Dimension(imageIcon.getIconWidth(), imageIcon.getIconHeight()));
		imagePanel.add(button);		
	}
}
