import java.awt.image.renderable.ParameterBlock;
import javax.media.jai.JAI;
import javax.media.jai.PlanarImage;
import javax.media.jai.Histogram;

/** Class that builds color histogram over the entire image
 *  
 *  @version 1.0 01.06.06
 *  @author Vegard Kosmo
 *
 */
public class GlobalHistogram {
	private PlanarImage destination;
	private ParameterBlock parameterBlock;
	
	/** Standard constructor
	 * 
	 */
	public GlobalHistogram(){
		
	}
	
	/** Method to make a global color histogram for the entire image
	 * 
	 *  @param planarImage The input image to build color histogram on
	 *  @return histogram Returns the generated color histogram
	 */
	public Histogram createGlobalHistogram(PlanarImage planarImage){
				
		// The histogram values
		int[] bins = {64, 64, 64};
		double[] low = {0,0,0};
		double[] high = {256,256,256};

		// Sets the parameter block properties
		parameterBlock = new ParameterBlock();
		parameterBlock.addSource(planarImage);
		parameterBlock.add(null); 
		parameterBlock.add(1);
		parameterBlock.add(1);
		parameterBlock.add(bins);
		parameterBlock.add(low);
		parameterBlock.add(high);
		
		// Returns the generated histogram
		destination = (PlanarImage) JAI.create("histogram", parameterBlock, null);
		return (Histogram)destination.getProperty("histogram");
	}
}
