import javax.media.jai.Histogram;

/** Class that compares two global color histograms
 * 
 *  @version 1.0 01.06.06
 * 	@author Vegard Kosmo
 * 	@see Videosegmenter
 *
 */
public class GlobalHistogramComparator{
	// Main class reference
	private Videosegmenter main;
	
	/** Standard constructor 
	 * 
	 *  @param main Main class reference
	 */
	public GlobalHistogramComparator(Videosegmenter main){
		this.main = main;
	}
	
	/** Method that compares two global color histograms and returns 'true' if the difference is larger than the threshold
	 *  
	 *  @param histogram1 The first histogram for comparing
	 *  @param histogram2 The second histogram for comparing
	 *  @return true/false Returns 'true' if the difference is larger than the threshold
	 */
	public boolean compareGlobalHistograms(Histogram histogram1, Histogram histogram2){
		
		// Sums up the histogram's difference values
		int[] sd = new int[3];
		
		for (int i = histogram1.getNumBins(0)-1;i >= 0; i--) {
			sd[0] = sd[0] + Math.abs(histogram1.getBinSize(0,i) - histogram2.getBinSize(0,i));
			sd[1] = sd[1] + Math.abs(histogram1.getBinSize(1,i) - histogram2.getBinSize(1,i));
			sd[2] = sd[2] + Math.abs(histogram1.getBinSize(2,i) - histogram2.getBinSize(2,i));
		}

		int sdFinal = sd[0] + sd[1] + sd[2];
				
		// Compares the sum with the threshold given from the algorithm class
		if(sdFinal > ((main.algorithm.getGlobalThreshold())*(main.getThresholdFactor()))){		
			return true;  
		}
		
		else{
			return false;
		}
	}
}
