import java.awt.Graphics2D;
import java.awt.Image;
import java.awt.image.*;

/** Class that resizes the images to be shown in the video shot window after segmentation
 *  
 *  @version 1.0 01.06.06
 *  @author Vegard Kosmo
 *
 */
public class ImageResizer {
	private BufferedImage smallImage;
	private int scalingFactor;
	
	/** Standard constructor
	 * 
	 */
	public ImageResizer(){
		
	}
	
	/** Method that resizes a planar image
	 * 
	 *  @param largeImage The input image to be resized
	 *  @param imageWidth The width of the image		
	 *  @param imageHeight The height of the image	
	 *  @return smallImage Returns the resized image
	 */
	public BufferedImage resizeImage(Image largeImage, int imageWidth, int imageHeight){
				
		// Sets the image scaling factor
		if(imageHeight < 200){
			scalingFactor = 2;
		}
		else if(imageHeight > 400){
			scalingFactor = 4;
		}
		else{
			scalingFactor = 3;
		}
		
		// Scales down the image and converts it into a BufferedImage
		Image scaledImage = largeImage.getScaledInstance(imageWidth/scalingFactor, imageHeight/scalingFactor, Image.SCALE_AREA_AVERAGING);
		int w = scaledImage.getWidth(null);
	    int h = scaledImage.getHeight(null);
	    smallImage = new BufferedImage(w, h, BufferedImage.TYPE_INT_RGB);
	    Graphics2D g = smallImage.createGraphics();
	    g.drawImage(scaledImage, 0, 0, null);
	    g.dispose();
						
		// Returns the scaled result		
		return smallImage;
	}
}
