import java.awt.Rectangle;
import java.awt.image.renderable.ParameterBlock;
import javax.media.jai.JAI;
import javax.media.jai.PlanarImage;
import javax.media.jai.Histogram;
import javax.media.jai.ROI;
import javax.media.jai.ROIShape;

/** Class that builds color histograms over 9 different regions of the image
 *  
 *  @version 1.0 01.06.06
 *  @author Vegard Kosmo
 *
 */
public class LocalHistogram{
	private PlanarImage destination;
	private ParameterBlock parameterBlock;
	private Histogram[] histogramList;
	private Rectangle r;
	private ROI roi;
	
	/** Standard constructor. Builds a histogram list for storing the region histograms
	 * 
	 */
	public LocalHistogram(){
		histogramList = new Histogram[9];
	}
		
	/** Method to make color histograms for 9 different regions of the image, and return a list containing these histograms
	 *  
	 *  @param planarImage The image to build histograms on
	 *  @return histogramList The list of the generated histograms
	 */
	public Histogram[] createLocalHistogram(PlanarImage planarImage){
		
		// The histogram values
		int[] bins = {64, 64, 64};
		double[] low = {0,0,0};
		double[] high = {256,256,256};
		
		// Image stats
		int regionWidth = (planarImage.getWidth()/3);
		int regionHeight = (planarImage.getHeight()/3);
		
		// A loop that generates one histogram for each Region of Interest (ROI)
		for(int i = 0; i < 9; i++){
			/* Sets the right region of interest
			 * 1 | 2 | 3
			 *---|---|---
			 * 4 | 5 | 6
			 *---|---|---
			 * 7 | 8 | 9 
			 */ 
			switch(i){
				case 0: r = new Rectangle(0, 0, regionWidth, regionHeight); break;
				case 1: r = new Rectangle(regionWidth, 0, regionWidth, regionHeight); break;
				case 2: r = new Rectangle(regionWidth*2, 0, regionWidth, regionHeight); break;
				case 3: r = new Rectangle(0, regionHeight, regionWidth, regionHeight); break;
				case 4: r = new Rectangle(regionWidth, regionHeight, regionWidth, regionHeight); break;
				case 5: r = new Rectangle(regionWidth*2, regionHeight, regionWidth, regionHeight); break;
				case 6: r = new Rectangle(0, regionHeight*2, regionWidth, regionHeight); break;
				case 7: r = new Rectangle(regionWidth, regionHeight*2, regionWidth, regionHeight); break;
				case 8: r = new Rectangle(regionWidth*2, regionHeight*2, regionWidth, regionHeight); break;				
			}
						
			roi = new ROIShape(r);
			
			// Sets the parameter block properties
			parameterBlock = new ParameterBlock();
			parameterBlock.addSource(planarImage);
			parameterBlock.add(roi);  
			parameterBlock.add(1);
			parameterBlock.add(1);
			parameterBlock.add(bins);
			parameterBlock.add(low);
			parameterBlock.add(high);
						
			destination = (PlanarImage) JAI.create("histogram", parameterBlock, null);
			histogramList[i] = (Histogram)destination.getProperty("histogram");
		}
		
		// Returns the list of generated histograms
		return histogramList;
	}
}
