import java.awt.*;
import java.awt.FileDialog;
import javax.swing.*;

/** Main class for the video segmenter
 *  
 *  This class builds all the components of the video segmenter application
 *  
 *  @version 1.0 01.06.06
 *  @author Vegard Kosmo
 *	@see     GUI
 *	@see     Algorithm
 *	@see     Mediaplayer
 *  @see     FrameSelector
 */

public class Videosegmenter{
	// GUI variables	 
	public GUI gui;
	public JFrame videoWindow;
	public JFrame shotWindow;

	//Algorithm variables	 
	public Algorithm algorithm;
	public double thresholdFactor;
	public GlobalHistogramComparator globalHistogramComparator;
	public LocalHistogramComparator localHistogramComparator;
	public EdgeHistogramComparator edgeHistogramComparator;
	public ImageResizer imageResizer; 
	public boolean algorithm1;
	public boolean algorithm2;
	public boolean algorithm3;

	// Media player variables	 
	public Mediaplayer mediaPlayer;
	private FileDialog fileDialog;
	private String fileName;
	
	// Frame selector variables	 
	public FrameSelector frameSelector;
	
	/** Standard constructor
	 * 
	 *  Sets the algorithm variables and threshold factor
	 */
	public Videosegmenter(){
		algorithm1 = true;
		algorithm2 = false;
		algorithm3 = false;
		thresholdFactor = 2.5;
	}

	/** Method to build the GUI
	 * 
	 *  Builds the graphical user interface of the application
	 */	
	private void buildGUI(){
		gui = new GUI(this);
		Dimension videoDimension = new Dimension(400, 480);
						
		// The video window
		videoWindow = gui.videoWindow(videoDimension);
		videoWindow.setJMenuBar(gui.videoMenu());		
		videoWindow.setVisible(true); 
		
		// The shot window
		shotWindow = gui.shotWindow();
		shotWindow.setVisible(false); 
	}

	/** Method to open a video file and create a media player
	 * 
	 */
	public void openFile(){
		mediaPlayer = new Mediaplayer(this, gui);
		fileDialog = null;
		
		// Creates a file dialog to open a video file
		if (fileDialog == null){
			fileDialog = new FileDialog(videoWindow, "Open File", FileDialog.LOAD);
		}
		
		// Shows the window for file opening
		fileDialog.setVisible(true);
		
		// Builds a String with directory and file name
		if (fileDialog.getFile() != null){
			fileName = fileDialog.getDirectory() + fileDialog.getFile();
			
			// Creates a media player that opens the video file with the given file name
			mediaPlayer.buildPlayer("file:" + fileName);
		}
	}

	/** Method to shut down the application
	 * 
	 * 
	 */
	public void exit(){
		shotWindow.dispose();
		videoWindow.dispose();
		System.exit(0);
	}
		 
	/**
	 * Method to change the algorithm boolean value
	 * 
	 * @param id The id of the algorithm	
	 * 
	 */
	public void changeValues(int id){
		// Changes the boolean values in the 'Algorithm' class
		if(id == 1){
			algorithm1 = true;
			algorithm2 = false;
			algorithm3 = false;
		}
		
		else if(id == 2){
			algorithm1 = false;
			algorithm2 = true;
			algorithm3 = true;
		}
		
		else if(id == 3){
			algorithm1 = false;
			algorithm2 = false;
			algorithm3 = true;
		}
	}

	/** Method to change the image comparing threshold value
	 * 
	 *  @param id The id of the threshold to be changed
	 */
	public void changeThreshold(int id){
		// Changes the threshold value in the 'ImageComparing' class
		if(id == 1){
			thresholdFactor = 2.2;
		}
		
		else if(id == 2){
			thresholdFactor = 2.5;
		}
		
		else if(id == 3){
			thresholdFactor = 2.8;
		}
	}

	/** Method to return the threshold factor value
	 * 
	 *  @return The activated threshold factor
	 */
	public double getThresholdFactor(){
		return thresholdFactor;
	}
	
	/** Method to build the video shot detection algorithm
	 * 
	 */
	public void buildAlgorithm(){
		algorithm = new Algorithm(this);
	}
	
	/** Method to build the frame selector
	 * 
	 */
	public void buildFrameSelector(){
		frameSelector = new FrameSelector(mediaPlayer);
	}
	
	/** Method to build the global histogram comparator
	 * 
	 */
	public void buildGlobalHistogramComparator(){
		globalHistogramComparator = new GlobalHistogramComparator(this);		
		imageResizer = new ImageResizer();
	}
	
	/** Method to build the local histogram comparator
	 * 
	 */
	public void buildLocalHistogramComparator(){
		localHistogramComparator = new LocalHistogramComparator(this);
		imageResizer = new ImageResizer();
	}

	/** Method to build the edge histogram comparator
	 * 
	 */
	public void buildEdegHistogramComparator(){
		edgeHistogramComparator = new EdgeHistogramComparator(this);
		imageResizer = new ImageResizer();
	}

	/** Main method to start the application
	 * 
	 *  @param args The standard args argument of the main method
	 */
	public static void main(String args[]){
		Videosegmenter videosegmenter = new Videosegmenter();
		videosegmenter.buildGUI();
		videosegmenter.buildAlgorithm();
	}
}
